using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace IDP
{
    public class _3DSurface: I3DSurface
    {
        private const long DF_POINT_EXTREME = 1000000;
                
        private const double RATIO = 0.99999;
        
        double a, aa, b, bb, r, sx, sy, hx, hy, s, x, y, nMax, u, v, z;
        
        private Point[] m_aPointsExt = new Point[DF_POINT_EXTREME];

        private static long lIdx = 0;

        private enum _3DSurfaceTypes
        {
            _3DSurfceTypeCos = 1
            , _3DSurfceTypeExp
            , _3DSurfaceTypesin
            , _3DSurfceTypeCos2
            , _3DSurfceTypeExt
        }

        public long _3DSurfceTypeCos
        {
            get
            {
                return (long)_3DSurfaceTypes._3DSurfceTypeCos;
            }
        }

        public long _3DSurfceTypeExp
        {
            get
            {
                return (long)_3DSurfaceTypes._3DSurfceTypeExp;
            }
        }

        public long _3DSurfaceTypesin
        {
            get
            {
                return (long)_3DSurfaceTypes._3DSurfaceTypesin;
            }
        }

        public long _3DSurfceTypeCos2
        {
            get
            {
                return (long)_3DSurfaceTypes._3DSurfceTypeCos2;
            }
        }

        public long _3DSurfceTypeExt
        {
            get
            {
                return (long)_3DSurfaceTypes._3DSurfceTypeExt;
            }
        }

        public long _3DSurfacePointLim
        { 
            get
            {
                return DF_POINT_EXTREME;
            }

        }

        public _3DSurface()
        {
            lIdx = 0;
        }

        public bool RenderBitmap(Bitmap oBmp)
        {
            if (lIdx < (DF_POINT_EXTREME))
            {
                if ((m_aPointsExt[lIdx].X != 0) && (m_aPointsExt[lIdx].Y != 0))
                {
                    oBmp.SetPixel(m_aPointsExt[lIdx].X
                                    , m_aPointsExt[lIdx].Y
                                    , System.Drawing.Color.White);
                }
                lIdx++;
            }
            return true;
        }

        private double pvGet3Dimension(_3DSurfaceTypes lSurfaceType)
        {
            double dblRetValue = 0.0;
            switch (lSurfaceType)
            {
                case _3DSurfaceTypes._3DSurfceTypeCos:
                    dblRetValue = 10 * Math.Cos(r / 5) + b;
                    return dblRetValue;
                case _3DSurfaceTypes._3DSurfceTypeExp:
                    dblRetValue = 75 * Math.Exp(-r * r / 600) + b;
                    return dblRetValue;
                case _3DSurfaceTypes._3DSurfaceTypesin:
                    dblRetValue = 125 * Math.Sin(r / 5) / r + b;
                    return dblRetValue;
                case _3DSurfaceTypes._3DSurfceTypeCos2:
                    dblRetValue = 10 * Math.Cos(x / 10) * Math.Cos(y / 10) + b;
                    return dblRetValue;
                case _3DSurfaceTypes._3DSurfceTypeExt:
                    dblRetValue = x * y * (x - y) * (x + y) / (1000 * r) + b;
                    return dblRetValue;
                default:
                    goto case _3DSurfaceTypes._3DSurfceTypeCos;
            }
        }

        private bool pvCalculateSurface(_3DSurfaceTypes lSurfaceType)
        {
            long lIdx = 0;
            hy = sy / 2;
            hx = sx / 2;
            s = Math.Sqrt(2) / 2;
            aa = hx * s;
            for (a = -aa; a <= (aa + 5 * s); a++)
            {
                nMax = -hy;
                bb = aa + a - 10 * s * (int)((a + Math.Abs(a)) / (10 * s));
                for (b = -bb; b <= bb + s * 4; b += 10 * s)
                {
                    x = s * (a + b);
                    y = s * (b - a);
                    z = b;
                    r = Math.Sqrt(x * x + y * y);
                    z = this.pvGet3Dimension(lSurfaceType);
                    if (z < nMax)
                    {
                    }
                    else
                    {
                        nMax = z;
                        u = hx + a;
                        v = hy + z * s / RATIO;

                        if (v < 0 || v > sy)
                        {
                        }
                        else
                        {
                            //fill array s u, v
                            if (lIdx < DF_POINT_EXTREME)
                            {
                                m_aPointsExt[lIdx].X = (int)u;
                                m_aPointsExt[lIdx].Y = (int)v;
                            }
                            lIdx++;
                        }
                    }
                }
            }
            return true;
        }

        private void pvClearArray()
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINT_EXTREME; lIdx++)
            {
                m_aPointsExt[lIdx].X = 0;
                m_aPointsExt[lIdx].Y = 0;
            }

        }




        public bool I3DGenerateGraphic(long lWidth, long lHeight)
        {
            // Todo: Add code here.
            sx = lWidth;
            sy = lHeight;
            return true;
        }

        public bool I3DGenerate3DSurface(long l3DSurfaceType
                                          , long lWidth
                                          , long lHeight)
        {
            sx = lWidth;
            sy = lHeight;
            this.pvClearArray();
            switch ((_3DSurfaceTypes)l3DSurfaceType)
            {
                case _3DSurfaceTypes._3DSurfceTypeCos:
                    this.pvCalculateSurface((_3DSurfaceTypes)l3DSurfaceType);
                    break;
                case _3DSurfaceTypes._3DSurfceTypeExp:
                    this.pvCalculateSurface((_3DSurfaceTypes)l3DSurfaceType);
                    break;
                case _3DSurfaceTypes._3DSurfaceTypesin:
                    this.pvCalculateSurface((_3DSurfaceTypes)l3DSurfaceType);
                    break;
                case _3DSurfaceTypes._3DSurfceTypeCos2:
                    this.pvCalculateSurface((_3DSurfaceTypes)l3DSurfaceType);
                    break;
                case _3DSurfaceTypes._3DSurfceTypeExt:
                    this.pvCalculateSurface((_3DSurfaceTypes)l3DSurfaceType);
                    break;
                default:
                    goto case _3DSurfaceTypes._3DSurfceTypeCos;
            }
            return true;
        }

        public bool IRenderToScreen(System.Drawing.Graphics oGraphics
                                    , System.Drawing.Pen oPen)
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINT_EXTREME - 1; lIdx++)
            {
                if (m_aPointsExt[lIdx].X != 0 && m_aPointsExt[lIdx].Y != 0)
                {
                    Rectangle oDot =
                        new Rectangle(m_aPointsExt[lIdx].X, m_aPointsExt[lIdx].Y, 1, 1);
                    oGraphics.DrawRectangle(oPen, oDot);
                }
            }
            return true;
        }

        public bool IRenderToScreenDelayed(System.Drawing.Graphics oGraphics
                                    , System.Drawing.Pen oPen)
        {
            if (lIdx < (DF_POINT_EXTREME))
            {
                Point oPoint = new Point();               
                oPoint = m_aPointsExt[lIdx];

                Rectangle oDot =
                    new Rectangle(oPoint.X, oPoint.Y, 1, 1);
                if ((oPoint.X != 0) && (oPoint.Y != 0))
                {
                    oGraphics.DrawRectangle(oPen, oDot);
                }
                lIdx++;
            }
            return true;
        }
 
        public bool IRenderToBitmapDelayed(Bitmap oBmp)
        {
            if (lIdx < (DF_POINT_EXTREME))
            {
                if ((m_aPointsExt[lIdx].X != 0) && (m_aPointsExt[lIdx].Y != 0))
                {
                    oBmp.SetPixel(m_aPointsExt[lIdx].X
                                    , m_aPointsExt[lIdx].Y
                                    , System.Drawing.Color.White);
                }
                lIdx++;
            }
            return true;
        }

        public bool IRenderToBitmap(Bitmap oBmp)
        {
            long lIdx = 0;
            for (lIdx = 0; lIdx < DF_POINT_EXTREME; lIdx++)
            {
                if (m_aPointsExt[lIdx].X != 0 && m_aPointsExt[lIdx].Y != 0)
                {
                    oBmp.SetPixel(m_aPointsExt[lIdx].X, m_aPointsExt[lIdx].Y, System.Drawing.Color.White);
                }
            }
            return true;
        }

        public bool IRenderToFile(string sFileName)
        {
            // Todo: Add code here.
            return true;
        }

       

    }
}
